@@ -17,6 +17,7 @@
 from __future__ import annotations
 
 import os
+from contextlib import nullcontext
 from pathlib import Path
 from urllib.parse import urlparse
 
@@ -24,9 +25,7 @@ import structlog
 from git import Repo
 from git.exc import BadName, GitCommandError, NoSuchPathError
 
-from airflow.dag_processing.bundles.base import (
-    BaseDagBundle,
-)
+from airflow.dag_processing.bundles.base import BaseDagBundle
 from airflow.exceptions import AirflowException
 from airflow.providers.git.hooks.git import GitHook
 
@@ -53,7 +52,7 @@ class GitDagBundle(BaseDagBundle):
         *,
         tracking_ref: str,
         subdir: str | None = None,
-        git_conn_id: str = "git_default",
+        git_conn_id: str | None = None,
         repo_url: str | None = None,
         **kwargs,
     ) -> None:
@@ -75,22 +74,26 @@ class GitDagBundle(BaseDagBundle):
             repo_path=self.repo_path,
             versions_path=self.versions_dir,
             git_conn_id=self.git_conn_id,
-            repo_url=self.repo_url,
         )
 
         self._log.debug("bundle configured")
+        self.hook: GitHook | None = None
         try:
-            self.hook = GitHook(git_conn_id=self.git_conn_id, repo_url=self.repo_url)
+            self.hook = GitHook(git_conn_id=git_conn_id or "git_default")
+        except Exception as e:
+            self._log.warning("Could not create GitHook", conn_id=git_conn_id, exc=e)
+
+        if not repo_url and self.hook and self.hook.repo_url:
             self.repo_url = self.hook.repo_url
-            self._log.debug("repo_url updated from hook", repo_url=self.repo_url)
-        except AirflowException as e:
-            self._log.warning("Could not create GitHook", conn_id=self.git_conn_id, exc=e)
+            self._log.debug("repo_url updated from hook")
 
     def _initialize(self):
         with self.lock():
-            with self.hook.configure_hook_env():
+            cm = self.hook.configure_hook_env() if self.hook else nullcontext()
+            with cm:
                 self._clone_bare_repo_if_required()
                 self._ensure_version_in_bare_repo()
+            self.bare_repo.close()
 
             self._clone_repo_if_required()
             self.repo.git.checkout(self.tracking_ref)
@@ -102,6 +105,7 @@ class GitDagBundle(BaseDagBundle):
                 self.repo.head.reset(index=True, working_tree=True)
             else:
                 self.refresh()
+            self.repo.close()
 
     def initialize(self) -> None:
         if not self.repo_url:
@@ -134,7 +138,7 @@ class GitDagBundle(BaseDagBundle):
                     url=self.repo_url,
                     to_path=self.bare_repo_path,
                     bare=True,
-                    env=self.hook.env,
+                    env=self.hook.env if self.hook else None,
                 )
             except GitCommandError as e:
                 raise AirflowException("Error cloning repository") from e
@@ -159,7 +163,8 @@ class GitDagBundle(BaseDagBundle):
         )
 
     def get_current_version(self) -> str:
-        return self.repo.head.commit.hexsha
+        with self.repo as repo:
+            return repo.head.commit.hexsha
 
     @property
     def path(self) -> Path:
@@ -177,18 +182,20 @@ class GitDagBundle(BaseDagBundle):
 
     def _fetch_bare_repo(self):
         refspecs = ["+refs/heads/*:refs/heads/*", "+refs/tags/*:refs/tags/*"]
-        if self.hook.env:
-            with self.bare_repo.git.custom_environment(GIT_SSH_COMMAND=self.hook.env.get("GIT_SSH_COMMAND")):
-                self.bare_repo.remotes.origin.fetch(refspecs)
-        else:
+        cm = nullcontext()
+        if self.hook and (cmd := self.hook.env.get("GIT_SSH_COMMAND")):
+            cm = self.bare_repo.git.custom_environment(GIT_SSH_COMMAND=cmd)
+        with cm:
             self.bare_repo.remotes.origin.fetch(refspecs)
+            self.bare_repo.close()
 
     def refresh(self) -> None:
         if self.version:
             raise AirflowException("Refreshing a specific version is not supported")
 
         with self.lock():
-            with self.hook.configure_hook_env():
+            cm = self.hook.configure_hook_env() if self.hook else nullcontext()
+            with cm:
                 self._fetch_bare_repo()
                 self.repo.remotes.origin.fetch(
                     ["+refs/heads/*:refs/remotes/origin/*", "+refs/tags/*:refs/tags/*"]
@@ -199,6 +206,7 @@ class GitDagBundle(BaseDagBundle):
                 else:
                     target = self.tracking_ref
                 self.repo.head.reset(target, index=True, working_tree=True)
+                self.repo.close()
 
     @staticmethod
     def _convert_git_ssh_url_to_https(url: str) -> str:
@@ -210,30 +218,58 @@ class GitDagBundle(BaseDagBundle):
         return f"{domain}/{repo_path}"
 
     def view_url(self, version: str | None = None) -> str | None:
+        """
+        Return a URL for viewing the DAGs in the repository.
+
+        This method is deprecated and will be removed when the minimum supported Airflow version is 3.1.
+        Use `view_url_template` instead.
+        """
         if not version:
             return None
-        url = self.repo_url
-        if not url:
+        template = self.view_url_template()
+        if not template:
             return None
+        if not self.subdir:
+            # remove {subdir} from the template if subdir is not set
+            template = template.replace("/{subdir}", "")
+        return template.format(version=version, subdir=self.subdir)
+
+    def view_url_template(self) -> str | None:
+        if hasattr(self, "_view_url_template") and self._view_url_template:
+            # Because we use this method in the view_url method, we need to handle
+            # backward compatibility for Airflow versions that doesn't have the
+            # _view_url_template attribute. Should be removed when we drop support for Airflow 3.0
+            return self._view_url_template
+
+        if not self.repo_url:
+            return None
+
+        url = self.repo_url
         if url.startswith("git@"):
             url = self._convert_git_ssh_url_to_https(url)
         if url.endswith(".git"):
             url = url[:-4]
+
         parsed_url = urlparse(url)
         host = parsed_url.hostname
         if not host:
             return None
+
         if parsed_url.username or parsed_url.password:
             new_netloc = host
             if parsed_url.port:
                 new_netloc += f":{parsed_url.port}"
             url = parsed_url._replace(netloc=new_netloc).geturl()
+
         host_patterns = {
-            "github.com": f"{url}/tree/{version}",
-            "gitlab.com": f"{url}/-/tree/{version}",
-            "bitbucket.org": f"{url}/src/{version}",
+            "github.com": f"{url}/tree/{{version}}",
+            "gitlab.com": f"{url}/-/tree/{{version}}",
+            "bitbucket.org": f"{url}/src/{{version}}",
         }
+
         for allowed_host, template in host_patterns.items():
             if host == allowed_host or host.endswith(f".{allowed_host}"):
+                if self.subdir:
+                    return f"{template}/{self.subdir}"
                 return template
         return None
